------------------------------------------------------------------------------
-- POLYTECHNIQUE MONTREAL
-- ELE3311 - Systemes logiques programmables 
-- 
-- Module Name:   spi_slave_tb
-- Description:   Banc d'essai du peripherique SPI
--                Les descriptions de ce fichier sont majoritairement
--                non-synthetisables.
--
-- Mise a jour (synchroniser avec TB_VERSION):
-- . 2021-03-19 : Creation
--
------------------------------------------------------------------------------



library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity spi_slave_tb is
generic (
  TB_VERSION              : string := "SPI_TB v2021-03-19";
  SHORT_SIM               : boolean := true;
  DATA_WIDTH              : integer := 12
);
end spi_slave_tb;


architecture testbench of spi_slave_tb is

  component spi_slave is
  generic (
    SPI_CPOL              : integer := 0;
    SPI_CPHA              : integer := 0;
    DATA_WIDTH            : integer := 12
  );
  port (
    sclk_i                : in    std_logic;
    mosi_i                : in    std_logic;
    miso_o                : out   std_logic;
    ss_n_i                : in    std_logic;
    rx_data_o             : out   std_logic_vector(DATA_WIDTH-1 downto 0);
    rx_data_valid_o       : out   std_logic;
    tx_data_i             : in    std_logic_vector(DATA_WIDTH-1 downto 0);
    tx_data_rdy_o         : out   std_logic
  );
  end component;

  -- Serielle
  signal sclk                     : std_logic := '0';
  signal mosi                     : std_logic := 'Z';
  signal miso                     : std_logic;
  signal ss_n                     : std_logic := '1';

  -- Parallele
  signal rx_data                  : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal rx_data_valid            : std_logic;
  signal tx_data                  : std_logic_vector(DATA_WIDTH-1 downto 0) := (others => '0');
  signal tx_data_rdy              : std_logic;

  -- Periode d'horloge
  constant sclk_period            : time := 50 ns; -- 20 MHz

  procedure spi_agent_send_word (
    variable data           : in  unsigned(DATA_WIDTH-1 downto 0);
    variable last           : in  boolean;  -- raise ss_n if true
    signal sclk             : out std_logic;
    signal mosi             : out std_logic;
    signal ss_n             : out std_logic
  ) is
    variable word_width     : integer := 12;
  begin
    ss_n             <= '0';

    for data_bit_select in word_width-1 downto 0 loop
      mosi             <= data(data_bit_select);
      wait for sclk_period/2;
      sclk             <= '1';
      wait for sclk_period/2;
      sclk             <= '0';
    end loop;

    if last then
      mosi             <= 'Z';
      wait for sclk_period/2;
      ss_n             <= '1';
    end if;

  end spi_agent_send_word;
  
begin

  -- Instantiation du composant a verifier - Unit Under Test (UUT)
  UUT: spi_slave
  generic map (
    SPI_CPOL              => 0,
    SPI_CPHA              => 0,
    DATA_WIDTH            => DATA_WIDTH
  )
  port map (
    sclk_i                => sclk,
    mosi_i                => mosi,
    miso_o                => miso,
    ss_n_i                => ss_n,
    rx_data_o             => rx_data,
    rx_data_valid_o       => rx_data_valid,
    tx_data_i             => tx_data,
    tx_data_rdy_o         => tx_data_rdy
  );


  -- Stimulus process
  driver: process
    variable spi_data                 : unsigned(DATA_WIDTH-1 downto 0);
    variable spi_last_data            : boolean := true;
  begin
    wait for 100*sclk_period;

    report "***** Section a inclure au rapport (DEBUT) " & TB_VERSION & " *****";
    --------------------------------------------------------------------------
    report "Transmettre 0x123 - Recevoir 0xCBA";
    --------------------------------------------------------------------------
    spi_data        := X"123";
    spi_last_data   := true;
    tx_data         <= X"CBA";
    spi_agent_send_word(
      data => spi_data, last => spi_last_data,
      sclk => sclk, mosi => mosi, ss_n => ss_n
    );

    wait for 20*sclk_period;

    --------------------------------------------------------------------------
    report "Transmettre 0x456 0x789 - Recevoir 0xFED A55";
    report "Envoyer 0x456 0x789";
    --------------------------------------------------------------------------
    spi_data        := X"456";
    spi_last_data   := false;
    tx_data         <= X"FED";
    spi_agent_send_word(
      data => spi_data, last => spi_last_data,
      sclk => sclk, mosi => mosi, ss_n => ss_n
    );
    wait for sclk_period;
    spi_data        := X"789";
    spi_last_data   := true;
    tx_data         <= X"A55";
    spi_agent_send_word(
      data => spi_data, last => spi_last_data,
      sclk => sclk, mosi => mosi, ss_n => ss_n
    );
    
    wait for 20*sclk_period;
    -- Fin de la sequence de test
    wait;
  end process driver;

end testbench;
